/*
Google Music Frame :: Window

This file contains main window.

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using Gtk;
using Fenryxo.Logger;

namespace GMF{
	
	public class SettingsWindow : Gtk.Window{
		/// Settings dialog title
		private const string TITLE = _("Settings");
		private Application app;
		private Box box;
		private RadioButton proxy_buttons[3];
		private Entry proxy_entry;
		
		private static SettingsWindow win;
		
		public static void display(Application app){
			if(SettingsWindow.win == null){
				SettingsWindow.win = new SettingsWindow(app);
			}
			SettingsWindow.win.show_all();
			SettingsWindow.win.present();
		}
		
		/**
		 * Constructs new main window
		 * 
		 * @param app Application object
		 */
		private SettingsWindow(Application app){
			this.app = app;
			this.position = WindowPosition.CENTER;
			this.title = TITLE;
			try {
				this.icon = IconTheme.get_default().load_icon(this.app.icon, 48, 0);
			}
			catch(Error e){
				log_warning("Unable to load application icon.");
			}
			
			Label label;
			var notebook = new Notebook();
			this.box = new Gtk.VBox(false, 2);
			
			Box box = new Gtk.VBox(false, 2);
			/// Settings dialog tab
			notebook.append_page(box, new Label(_("General")));
			/// Settings dialog - tab General
			box.pack_start(this.create_bool_entry("multimedia_keys", _("Handle multimedia keys."), true), false, false);
			/// Settings dialog - tab General
			box.pack_start(this.create_bool_entry("pause_key", _("Use Pause (break) key to toggle play/pause"), false), false, false);
			/// Settings dialog - tab General
			box.pack_start(this.create_bool_entry("hide_when_playing", _("Hide window if close button is clicked while playing."), true), false, false);
			// TODO: option "Show tray icon or sound menu entry"
			
			
			
			box = new Gtk.VBox(false, 1);
			/// Settings dialog tab
			notebook.append_page(box, new Label(_("Connection")));
			/// Settings dialog - tab Connection
			label = new Label(_("Google Music Frame supports direct network connection or connection through a HTTP(S) proxy server. If you need to connect through a SOCKS proxy, you can use <a href=\"http://tsocks.sourceforge.net\">tsocks</a>.\n\nPlease restart Google Music frame to apply new proxy settings."));
			label.use_markup = true;
			label.wrap_mode = Pango.WrapMode.WORD;
			label.wrap = true;
			box.pack_start(label, false, false, 5);
						
			this.proxy_buttons = new RadioButton[3];
			/// Settings dialog - tab Connection
			this.proxy_buttons[0] = new RadioButton.with_label(null, _("Use direct connection"));
			box.pack_start(this.proxy_buttons[0], false, false, 5);
			/// Settings dialog - tab Connection
			this.proxy_buttons[1] = new RadioButton.with_label(this.proxy_buttons[0].get_group(), _("Use system settings"));
			box.pack_start(this.proxy_buttons[1], false, false, 5);
			/// Settings dialog - tab Connection
			this.proxy_buttons[2] = new RadioButton.with_label(this.proxy_buttons[0].get_group(), _("Use manual proxy settings (https://server:port)"));
			box.pack_start(this.proxy_buttons[2], false, false, 5);
			this.proxy_entry = new Entry();
			this.proxy_entry.text = this.app.config["proxy"] ?? "";
			this.proxy_entry.changed.connect(() => { this.app.config["proxy"] = this.proxy_entry.text; });
			box.pack_start(this.proxy_entry, false, false, 5);
			
			var use_proxy = this.app.config["use_proxy"] ?? "";
			if(use_proxy == "manual"){
				this.proxy_buttons[2].active = true;
				this.proxy_entry.sensitive = true;
			}
			else if(use_proxy == "" || use_proxy == "auto"){
				this.proxy_buttons[1].active = true;
				this.proxy_entry.sensitive = false;
			}
			else{
				this.proxy_buttons[0].active = true;
				this.proxy_entry.sensitive = false;
			}
			
			this.proxy_buttons[0].toggled.connect(this.proxy_toggled);
			this.proxy_buttons[1].toggled.connect(this.proxy_toggled);
			this.proxy_buttons[2].toggled.connect(this.proxy_toggled);
			
			
			
			
#if DEV_BUILD
			box = new Gtk.VBox(false, 2);
			label = new Label("User scripts are used to enhance Google Music web interface. // Settings not yet implemented.");
			label.wrap_mode = Pango.WrapMode.WORD;
			label.wrap = true;
			box.pack_start(label, false, false, 5);
			/// Settings dialog tab
			notebook.append_page(box, new Label(_("User scripts")));
#endif
			this.box.pack_start(notebook, false, true, 10);
			this.add(this.box);
			var bbox = new HButtonBox();
			var button = new Button.from_stock(Stock.APPLY);
			button.clicked.connect(() => { this.apply(); });
			bbox.add(button);
			button = new Button.from_stock(Stock.OK);
			button.clicked.connect(() => { this.apply(); this.close(); });
			bbox.add(button);
			button = new Button.from_stock(Stock.CANCEL);
			button.clicked.connect(() => { this.close(); });
			bbox.add(button);
			this.box.pack_start(bbox, false, false, 10);
			this.box.show_all();
		}
		
		private CheckButton create_bool_entry(string key, string label, bool default){
			var entry = new CheckButton.with_label(label);
			entry.active = this.app.config.bool(key, default);
			entry.toggled.connect(() => { this.set_bool(key, entry.active); });
			return entry;
		}
		
		
		private void proxy_toggled(ToggleButton button){
			log_debug("Proxy button toggled");
			if(button.active){
				if(button == this.proxy_buttons[0]){
					this.app.config["use_proxy"] = "none";
					this.proxy_entry.sensitive = false;
				}
				else if(button == this.proxy_buttons[1]){
					this.app.config["use_proxy"] = "auto";
					this.proxy_entry.sensitive = false;
				}
				else if(button == this.proxy_buttons[2]){
					this.app.config["use_proxy"] = "manual";
					this.proxy_entry.sensitive = true;
				}
			}
		}
		
		private void set_bool(string key, bool value){
			this.app.config[key] = value ? "on" : "off";
		}
		
		private void apply(){
			try{
				this.app.config.save();
			}
			catch(Error e){
				log_critical("Unable to save configuration: %s", e.message);
			}
			this.app.config.notify_changed();
		}
		
		
		public override bool delete_event(Gdk.Event event){
			SettingsWindow.win = null;
			return false;
		}
		
		public void close(){
			SettingsWindow.win = null;
			this.destroy();
		}
	}
}
