/*
Google Music Frame

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using Fenryxo.Path;
using Fenryxo.Logger;

namespace GMF{
	const string ALBUM_ART_DIRNAME = "album_art";
	/**
	 * Partial storage abstraction class
	 */
	public class Storage : Object{
		private string path_name;
		
		/**
		 * Creates new storage identified by the id
		 * 
		 * @param id identifier of the storage
		 */
		public Storage(string id){
			this.path_name = id;
		}
		
		/**
		 * Stores remote album art to cache (if not cached yet) and return local path
		 * 
		 * @param Remota album art URI
		 * @return local path or null on failure
		 */
		public string? get_album_art(string? uri){
			if(uri == null) return null;
			/* Album art is stored locally to be passed to libnotify and Sound menu */
			var i = uri.last_index_of_char('/');
			var id = uri.substring(i+1);
			var f = this.get_default_cache_file(ALBUM_ART_DIRNAME + "/" + id + ".jpg");
			if(f.query_exists(null)){
				return f.get_path();
			}
			
			var session = WebKit.get_default_session();
			var msg = new Soup.Message("GET", uri);
			var status = session.send_message(msg);
			if(status != 200){
				return null;
			}
			
			unowned Soup.MessageBody body = msg.response_body;
			var dir = f.get_parent();
			if(!dir.query_exists(null)){
				try{
					dir.make_directory_with_parents(null);
				}
				catch(Error e){
					log_critical("Unable to create album art directory: %s", e.message);
				}
			}
			
			FileOutputStream stream;
			try{
				stream = f.create(FileCreateFlags.REPLACE_DESTINATION, null);
			}
			catch(Error e){
				log_critical("Unable to create album art file: %s", e.message);
				return null;
			}
			
			try{
				stream.write_all(body.data, null, null);
			}
			catch(IOError e){
				log_critical("Unable to store album art: %s", e.message);
				return null;
			}
			try{
				stream.close();
			}
			catch(IOError e){
				log_warning("Unable to close stream: %s", e.message);
			}
			
			return f.get_path();
		}
		
		public void purge_album_art(){
			var dir = this.get_default_cache_dir(ALBUM_ART_DIRNAME);
			log_debug(dir.get_path());
			try{
				var enumerator = dir.enumerate_children(FILE_ATTRIBUTE_STANDARD_NAME, 0);
				FileInfo file_info;
				while((file_info = enumerator.next_file ()) != null) {
					var f = dir.get_child(file_info.get_name());
					log_debug("Purge album art: %s", f.get_path());
					try{
						f.delete();
					}
					catch(Error e){
						log_warning("Unable to delete file %s: %s",
							f.get_path(), e.message);
					}
				}
			}
			catch(Error e){
				log_critical("Unable to purge album art: %s", e.message);
			}
		}
		
		/**
		 * Returns the default path of configuration file with given name
		 * 
		 * @param file Name of configuration file
		 * @return default configuration file
		 */
		public File? get_default_config_file(string file){
			return Fenryxo.Path.get_default_config_file(this.path_name, file);
		}
		
		/**
		 * Returns the default path of cache file with given name
		 * 
		 * @param file Name of cache file
		 * @return default cache file
		 */
		public File? get_default_cache_file(string file){
			return Fenryxo.Path.get_default_cache_file(this.path_name, file);
		}
		
		/**
		 * Returns the default path of cache dir with given name
		 * 
		 * @param file Name of cache file
		 * @return default cache file
		 */
		public File? get_default_cache_dir(string dir){
			return File.new_for_path(Environment.get_user_cache_dir())
			.get_child(this.path_name).get_child(dir);
		}
	}
}
